/*
 * TCSS 305Winter 2014
 * Assignment 6b-Tetris
 */

package view;
import java.awt.GridLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * This class creates the panel that holds the JOptionPane that lets user change or view 
 * controls.
 * 
 * @author Brian Dennis
 * @version 7 March 2014
 */
@SuppressWarnings("serial")
public class ChangeControlsPanel extends JPanel {
   
    /** integer used for spacing while creating struts for box layout. */
    private static final int STRUT_DISTANCE = 10;
    
    /** Array used to store the name of the original mappings of the key binds. Also used for 
     * property change keys.
     */
    private final String[] myNewControlArray;
    
    /** Array used to store the text contained in the Labels above the input fields. */
    private final String[] myStringLabelArray;
    
    /** Array used to store the actual labels located above the input fields. */
    private final JLabel[] myLabelArray;
    
    /** Array used to store the JTextFields. */
    private JTextField[] myFieldArray;
    
    /**
     * The constructor which initializes the various variables. And calls 
     * setupControlDialogBox() to start the process.
     */
    public ChangeControlsPanel() {
       
        super();
        myStringLabelArray = new String[] {"Move Left: ", "Move Right: ", "Move Down: ", 
            "Rotate Right: ", "Rotate Left: ", "Drop Block: ", "Pause: "};
        myLabelArray = new JLabel[myStringLabelArray.length];
        myFieldArray = new JTextField[myStringLabelArray.length];
        myNewControlArray = new String[] {"A", "D", "S", "RIGHT", "LEFT", "SPACE", "P"};
        
        fieldsIntoArray();
        setupControlDialogBox();
    }
    
    /**
     * This method adds created fields into an array, making a method call to createFields().
     */
    private void fieldsIntoArray() {
        
        for (int i = 0; i < myNewControlArray.length; i++) {
            
            myFieldArray[i] = createFields(myNewControlArray[i]);
        }
    }
    
    /**
     * This method allows a JTextField to be created, adding an anonymous KeyListener to each 
     * field.
     * @param theControl which is the name contained on each label
     * @return the JTextField being created
     */
    private JTextField createFields(final String theControl) {
        final JTextField field = new JTextField(theControl);
        field.addKeyListener(new KeyAdapter() {
            /** 
             * @param theAction the action that corresponds to what the user does
             */
            public void keyPressed(final KeyEvent theEvent) {
                final int keyCode = theEvent.getKeyCode();
                field.setEditable(false); 
                field.setText(KeyEvent.getKeyText(keyCode)); 
                
                fireChanges(keyCode, field);
            }
        });
     
        return field;
    }
    
    /**
     * This method creates the JPanel that holds the fields, and sets them to the correct 
     * state.
     */
    private void setupControlDialogBox() {

        for (int i = 0; i <= myLabelArray.length - 1; i++) {
            
            myLabelArray[i] = new JLabel(myStringLabelArray[i] + myNewControlArray[i]);
            final JPanel grid = new JPanel(new GridLayout(2, 1));
            add(grid);
            grid.add(myLabelArray[i]);
            grid.add(myFieldArray[i]);
            add(Box.createHorizontalStrut(STRUT_DISTANCE)); 
        }
    }

    /**
     * This method fires property changes associated with the current field. It also sets the 
     * associated JLabel to the key typed.
     * 
     * @param theKeyCode the key code of the event typed
     * @param theField the field that is being alters
     */
    private void fireChanges(final int theKeyCode, final JTextField theField) {
        
        for (int i = 0; i < myFieldArray.length; i++) {
            
            if (myFieldArray[i].hasFocus()) {
    
                firePropertyChange(myNewControlArray[i], myNewControlArray[i], theKeyCode);
                myLabelArray[i].setText(myStringLabelArray[i] + theField.getText());
            }
        }
    }
    
    /**
     * This method initiates the JOptionPane, and fires a propertyChange method if the 
     * controls have been changed.
     */
    public void initiateControlsDialog() {
        
        JOptionPane.showConfirmDialog(null, this, 
                           "Change or keep the current controls.  Press desired key in box to"
                           + " change key mappings.", 
                           JOptionPane.OK_CANCEL_OPTION);
    }
}