/*
 * TCSS 305Winter 2014
 * Assignment 6b-Tetris
 */

package view;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import model.Board;

/**
 * This class allows a KeyListener to be made. This class also manages the controls of the 
 * tetris pieces, and when a user changes the current controls.
 * 
 * @author Brian Dennis
 * @version 7 March 2014
 */
public class KeyListener extends KeyAdapter implements PropertyChangeListener {

    /** The initial key value of 'A'. */
    private static final int LEFT_KEY = 65;
    
    /** The initial key value of 'D'. */
    private static final int RIGHT_KEY = 68;
    
    /** The initial key value of 'S'. */
    private static final int DOWN_KEY = 83;
    
    /** The initial key value of 'RIGHT'. */
    private static final int ROTATE_R_KEY = 39;
    
    /** The initial key value of 'LEFT'. */
    private static final int ROTATE_L_KEY = 37;
    
    /** The initial key value of 'SPACE'. */
    private static final int DROP_KEY = 32;
    
    /** The integer that represents the value of the move left key. This value can change. */
    private int myLeftKeyValue = LEFT_KEY;
    
    /** The integer that represents the value of the move right key. This value can change. */
    private int myRightKeyValue = RIGHT_KEY;
    
    /** The integer that represents the value of the move down key. This value can change. */
    private int myDownKeyValue = DOWN_KEY;
    
    /** The integer that represents the value of the rotate clockwise key. This value can 
                                                                                    change. */
    private int myRorateRightKeyValue = ROTATE_R_KEY;
    
    /** The integer that represents the value of the rotate counterclockwise key. This value 
                                                                                can change. */
    private int myRotateLeftKeyValue = ROTATE_L_KEY;
    
    /** The integer that represents the value of the drop down key. This value can change. */
    private int myDropKeyValue = DROP_KEY;
        
    /** the reference to the board object of the game. */
    private final Board myBoard;
        
    /**
     * The constructor which initializes the variables to initial states.
     * 
     * @param theBoard the reference to the game board object
     */
    public KeyListener(final Board theBoard) {
         
        super();    
        myBoard = theBoard;
    }
        
    /**
     * Handles a key being typed.
     * 
     * @param theEvent The KeyEvent generated by the key.
     * {@inheritDoc}
     */
    @Override
    public void keyReleased(final KeyEvent theEvent) {
        
        if (theEvent.getKeyCode() == myLeftKeyValue) {
               
            myBoard.moveLeft();
                
        } else if (theEvent.getKeyCode() == myRightKeyValue) {

            myBoard.moveRight();
                
        } else if (theEvent.getKeyCode() == myDownKeyValue) {
                
            myBoard.moveDown();
                
        } else if (theEvent.getKeyCode() == myRorateRightKeyValue) {
                
            myBoard.rotateClockwise();
                
        } else if (theEvent.getKeyCode() == myRotateLeftKeyValue) {
                
            myBoard.rotateCounterclockwise();
                
        } else if (theEvent.getKeyCode() == myDropKeyValue) {
            
            myBoard.drop();
        }  
    }   
       
    /**
     * Handles the property changes when user changes the controls.
     * 
     * {@inheritDoc}
     */
    @Override
    public void propertyChange(final PropertyChangeEvent theEvent) {
         
        if ("A".equals(theEvent.getPropertyName())) {
    
            myLeftKeyValue = (int) theEvent.getNewValue();
                
        } else if ("D".equals(theEvent.getPropertyName())) {
         
            myRightKeyValue = (int) theEvent.getNewValue();
                
        } else if ("S".equals(theEvent.getPropertyName())) {
           
            myDownKeyValue = (int) theEvent.getNewValue();
                
        } else if ("RIGHT".equals(theEvent.getPropertyName())) {
     
            myRorateRightKeyValue = (int) theEvent.getNewValue();
                
        }  else if ("LEFT".equals(theEvent.getPropertyName())) {
 
            myRotateLeftKeyValue = (int) theEvent.getNewValue();
                
        } else if ("SPACE".equals(theEvent.getPropertyName())) {

            myDropKeyValue = (int) theEvent.getNewValue();
        }
    }
}