/*
 * TCSS 305Winter 2014
 * Assignment 6b-Tetris
 */

package view;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.Line2D;
import java.awt.geom.RoundRectangle2D;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JPanel;
import model.AbstractPiece;
import model.Board;
import model.ImmutablePiece;

/**
 * This class creates the panel that holds the screen that shows the next piece.
 * 
 * @author Brian Dennis
 * @version 7 March 2014
 */
@SuppressWarnings("serial")
public class NextPiecePanel extends JPanel implements Observer {

    /** the size of the currentBlock. */
    private static final int MY_BLOCK_SIZE = 25;
    
    /** the number representing how much curve the squares that make up the pieces will have.*/
    private static final int CURVE_NUMBER = 15;
    
    /** the number used to create the width and height of this panel. */
    private static final int MULTIPLIER = 6;
    
    /** the board of the game used to retrieve information from the game. */
    private final Board myBoard;
    
    
        
    /**
     * This constructor sets initial variable, size, and background color.
     * 
     * @param theBoard the board of the game
     */
    public NextPiecePanel(final Board theBoard) {
        
        super();
        myBoard = theBoard;

        setPreferredSize(new Dimension(MY_BLOCK_SIZE * MULTIPLIER, 
                                                                 MY_BLOCK_SIZE * MULTIPLIER));
        setBackground(Color.WHITE);
    }
    
    /**
     * Paints this panel on the screen with the specified Graphics object. Calls methods to 
     * draw components.
     * 
     * @param theGraphics The Graphics object.
     */
    @Override
    public void paintComponent(final Graphics theGraphics) {
        
        super.paintComponent(theGraphics);
        final Graphics2D g2 = (Graphics2D) theGraphics;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                                                           RenderingHints.VALUE_ANTIALIAS_ON);
        drawNextPiece(g2);
        drawGrid(g2);
    }

    /**
     * This method draws the next piece.
     * 
     * @param theGraphics the graphics object
     */
    private void drawNextPiece(final Graphics2D theGraphics) {
        
        final ImmutablePiece nextPiece = (AbstractPiece) myBoard.getNextPiece();
        final model.Point[] points = 
                                ((AbstractPiece) nextPiece).getRotations().get(0).getPoints(); 
            
        for (final model.Point pointOfPiece : points) {
                
            final Point point = new Point(pointOfPiece.getX(), pointOfPiece.getY());
            theGraphics.setPaint(((AbstractPiece) nextPiece).getColor());
            theGraphics.fill(new RoundRectangle2D.Double(MY_BLOCK_SIZE + (point.getX()) 
                                                                              * MY_BLOCK_SIZE, 
                             getHeight() - (point.getY() * MY_BLOCK_SIZE) - MY_BLOCK_SIZE * 2,
                                   MY_BLOCK_SIZE, MY_BLOCK_SIZE, CURVE_NUMBER, CURVE_NUMBER));
            theGraphics.setStroke(new BasicStroke(2));
            theGraphics.setPaint(Color.BLACK);
            theGraphics.draw(new RoundRectangle2D.Double(MY_BLOCK_SIZE + (point.getX()) 
                                                                              * MY_BLOCK_SIZE, 
                             getHeight() - (point.getY() * MY_BLOCK_SIZE) - MY_BLOCK_SIZE * 2,
                                   MY_BLOCK_SIZE, MY_BLOCK_SIZE, CURVE_NUMBER, CURVE_NUMBER));
        }
    }
    
    /**
     * this method draws the grid.
     * 
     * @param theGraphics the graphics object
     */
    private void drawGrid(final Graphics2D theGraphics) {
        
        theGraphics.setStroke(new BasicStroke(0));
        theGraphics.setPaint(Color.BLACK);
        for (int i = 0; i < getWidth(); i += MY_BLOCK_SIZE) {
            final Shape line = new Line2D.Float(i, 0, i, getHeight());
            theGraphics.draw(line);
        }

        for (int i = 0; i < getHeight(); i += MY_BLOCK_SIZE) {
            final Shape line = new Line2D.Float(0, i, getWidth(), i);
            theGraphics.draw(line);
        } 
    }
    
    /**
     * All this method does is update the graphics panel when called by the observable class.
     * {@inheritDoc}
     */
    @Override
    public void update(final Observable theBoard, final Object theArg) {
           
        repaint();
    }                 
}