/*
 * TCSS 305Winter 2014
 * Assignment 6b-Tetris
 */

package view;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.Timer;

/**
 * This class creates an Action for the pause button. Making the button usable through key 
 * bindings as well.
 * 
 * @author Brian Dennis
 * @version 13 March 2014
 */
@SuppressWarnings("serial")
public class PauseAction extends AbstractAction {
    
    /** The drawing board panel of the GUI. */
    private final JFrame myFrame;
    
    /** The panel that holds the visual representation of the tetris board. */
    private final TetrisPanel myTetris;
    
    /** the timer that controls the flow of the game. */
    private final Timer myMoveTimer;
    
    /** the boolean that allows the true and false state to be toggled. */
    private boolean myPausedStatus;
    
    /** The keyListener of the JFrame. used to remove them for pause status. */
    private final KeyListener myKeyListener;
    
    /**
     * Construct this Action for pause, setting the title of the button, and the initial 
     * state of the variables.
     * 
     * @param theFrame a JPanel to associate with this Action
     * @param theTetris theJPanel holding the tetris game
     * @param theMoveTimer the timer in charge of moving down the tetris pieces
     * @param theKeyListener the key listener for the GUI
     */
    public PauseAction(final JFrame theFrame, final TetrisPanel theTetris, 
                       final Timer theMoveTimer, final KeyListener theKeyListener) {
        
        super("Pause");
        myFrame = theFrame;
        myTetris = theTetris;
        myMoveTimer = theMoveTimer;
        myKeyListener = theKeyListener;
    }

    /**
     * This method toggles the myPausedStatus to true or false and then pauses the game 
     * depending on the status.
     * 
     * @param theEvent the event that occurs that initiates the action.
     */
    @Override
    public void actionPerformed(final ActionEvent theEvent) {
     
        myPausedStatus ^= true;
        
        if (myPausedStatus) {
            
            myMoveTimer.stop();
            myFrame.removeKeyListener(myKeyListener);
            myTetris.togglePause();
            myTetris.repaint();
            TetrisGUI.disableAllButPause();
            
        } else {
            
            myMoveTimer.start();
            myFrame.addKeyListener(myKeyListener);
            myTetris.togglePause();
            myTetris.repaint();
            TetrisGUI.enableAllButNewGame();
        }
    }
}