/*
 * TCSS 305Winter 2014
 * Assignment 6b-Tetris
 */

package view;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JPanel;
import javax.swing.Timer;
import model.Board;

/**
 * This class creates the panel that holds the JOptionPane that lets user change or view 
 * controls.
 * 
 * @author Brian Dennis
 * @version 13 March 2014
 */
@SuppressWarnings("serial")
public class ScorePanel extends JPanel implements Observer {

    /** HJow many points the player receives per line cleared. */
    private static final int POINTS_PER_LINE = 500;
                    
    /** How much the timer is decremented per level gained. */
    private static final int DECREMENT_TIMER = 25;
    
    /** the amount of lines needed for next levek. */
    private static final int LINES_TO_NEXT_LEVEL = 5;
    
    /** The x coordinate for displaying the text of the scores. */
    private static final int X_POSITION = 5;
    
    /** The number used to divide the height of the panel for text placement. */
    private static final int HEIGHT_DIVIDER = 3;
    
    /** The board representing the back end code of the game. */
    private final Board myBoard;
    
    /** The total score the player has. Defaulted to zero. */
    private int myScore;
    
    /** the total number of lines cleared. Defaulted to zero. */
    private int myLinesCleared;
    
    /** How many lines the player needs to clear to level up. Defaulted to zero. */
    private int myLinesToNextLevel = LINES_TO_NEXT_LEVEL;
    
    /** The speed of the timer which gets faster as the game levels. */
    private int mySpedUpTimer;
    
    /** The current level of the game. */
    private int myLevel = 1;
   
    /** The timer that moves the pieces down. */
    private final Timer myMoveTimer;
    
    /** An array of numbers used in this class. */
    private final int[] myMagicNumberArray = {3, 8};
    
    /**
     * This constructor initializes variables of the class and sets the background of this 
     * panel.
     * 
     * @param theBoard the board representing the game
     * @param theMoveTimer the timer that moves the game
     */
    public ScorePanel(final Board theBoard, final Timer theMoveTimer) {
        
        super();
        myBoard = theBoard;
        myMoveTimer = theMoveTimer;
        mySpedUpTimer = myMoveTimer.getDelay(); //initialize to 600
        setBackground(Color.ORANGE);
    }
    
    /**
     * Paints this panel on the screen with the specified Graphics object. Calls methods to 
     * draw components.
     * 
     * @param theGraphics The Graphics object.
     */
    @Override
    public void paintComponent(final Graphics theGraphics) {
        
        super.paintComponent(theGraphics);
        final Graphics2D g2 = (Graphics2D) theGraphics;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                                                           RenderingHints.VALUE_ANTIALIAS_ON);
        if (myBoard.getLastLinesRemoved() > 0) {
            
            myScore += POINTS_PER_LINE * myBoard.getLastLinesRemoved();
            myLinesCleared += myBoard.getLastLinesRemoved();
            myLinesToNextLevel -= myBoard.getLastLinesRemoved();
            
            if (myLinesToNextLevel <= 0) {
                
                mySpedUpTimer -= DECREMENT_TIMER;
                myLinesToNextLevel = LINES_TO_NEXT_LEVEL;
                myLevel += 1;
                myMoveTimer.setDelay(mySpedUpTimer);
            }
        }
    
        g2.drawString("LEVEL: " + myLevel, getWidth() / X_POSITION, 
                      X_POSITION * myMagicNumberArray[0]);                      // + 15 Ypos
        g2.drawString("Score: " + myScore, X_POSITION, 
                      getHeight() / HEIGHT_DIVIDER  + X_POSITION * 2);          // + 10 Ypos
        g2.drawString("Lines Cleared: " + myLinesCleared, X_POSITION, 
                      getHeight() / HEIGHT_DIVIDER + X_POSITION * X_POSITION);  // + 25 Ypos
        g2.drawString("Lines to next level: " + myLinesToNextLevel, 
                      X_POSITION, getHeight() / HEIGHT_DIVIDER + X_POSITION 
                      * myMagicNumberArray[1]);                                 // + 40 Ypos
    }
    
    /**
     * All this method does is update the graphics panel when called by the observable class.
     * {@inheritDoc}
     */
    @Override
    public void update(final Observable theBoard, final Object theArg) {
           
        repaint();
    }
}