/*
 * TCSS 305Winter 2014
 * Assignment 6b-Tetris
 */

package view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.WindowEvent;

import javax.swing.Action;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.Timer;

import model.Board;

/**
 * This class creates the basic Layout of the GUI which holds the tetris game and all of it's 
 * components. It also holds an inner class Action Listener which manages the actions of the 
 * timer, and certain JButtons.
 * 
 * @author Brian Dennis
 * @version 7 March 2014
 */
public class TetrisGUI implements FocusListener {

    /** A ToolKit for getting screen dimensions. */
    private static final Toolkit KIT = Toolkit.getDefaultToolkit();

    /** The Dimension of the screen. */
    private static final Dimension SCREEN_SIZE = KIT.getScreenSize();

    /** The width of the screen. */
    private static final int SCREEN_WIDTH = SCREEN_SIZE.width;

    /** The height of the screen. */
    private static final int SCREEN_HEIGHT = SCREEN_SIZE.height;
    
    /** the width of the Board class. */
    private static final int BOARD_WIDTH = 10;
    
    /** the height of the Board class. */
    private static final int BOARD_HEIGHT = BOARD_WIDTH * 2;
    
    /** The starting frequency of the timer's actions. */
    private static final int INITIAL_DELAY = 600;
    
    /** An array that holds the numbers used throughout this class. */
    private static final int[] MAGIC_NUMBER_ARRAY = {3, 4, 6, 65, 100, 200, 5};
    
    /** An array that holds all of the JButtons located on the east of the GUI. */
    private static JButton[] myButtonArray;
    
    /** The frame of the entire GUI. */
    private JFrame myFrame;
    
    /** The timer that controls the movement. */
    private Timer myMoveTimer;
    
    /** The listener that manages actions. */
    private Listener myActionListener;
    
    /** The listener that manages key monitoring. */
    private KeyListener myKeyListener;
    
    /** The separate listener for the pause button. */
    private PauseListener myPauseListener;
    
    /** The game Board that represents the tetris game. */
    private Board myBoard;
    
    /** The panel that that initiated the dialog that lets you change controls of the game. */
    private ChangeControlsPanel myChangeControls;
    
    /** The panel that contains the graphics of the tetris game. */
    private TetrisPanel myTetris;
    
    /** An Array that holds the names of the JButtons located on the east of the GUI. */
    private final String[] myButtonNameArray = {"Change Controls...", "Pause", "End Game", 
        "New Game", "Scoring Info..."};

    /** An array that holds the names of the sizing buttons, and action commands. */
    private final String[] myStringNamesArray = 
    {"move down", "Small", "Normal", "BIG", "Grid"};
    
    /**
     * The constructor of this class which calls setup() which in turn, sets the instance 
     * fields to reasonable defaults.
     */
    public TetrisGUI() {

        setup();
        changeControls();
    }

    /**
     * This method instantiates the main objects.  This method is called later if 
     * the user wants to play another game after it ends.
     */
    private void setup() {
        
        myFrame = new JFrame("TCSS 305 Tetris");
        myBoard = new Board(BOARD_WIDTH, BOARD_HEIGHT);
        myTetris = new TetrisPanel(myBoard);
        myActionListener = new Listener();
        myKeyListener = new KeyListener(myBoard);
        myMoveTimer = new Timer(INITIAL_DELAY, myActionListener);
        
        myFrame.addFocusListener(this);
        myFrame.addKeyListener(myKeyListener);
         
        myMoveTimer.setActionCommand(myStringNamesArray[0]); //move down
        myMoveTimer.start();
        
        myButtonArray = new JButton[myButtonNameArray.length];
    }
    /**
     * The start method of this class called by TetrisMain to start creating the 
     * JFrame and it's implementation. It does by setting states of the frame and then calling
     * a method to add the main components. Lastly it makes a call to changeControls() to 
     * display the game controls before the tetris game starts.
     */
    public void start() {  
        
        myFrame.setLocation(SCREEN_WIDTH / MAGIC_NUMBER_ARRAY[0]        //3
                        - myFrame.getWidth() / MAGIC_NUMBER_ARRAY[0],   //3
                        SCREEN_HEIGHT / MAGIC_NUMBER_ARRAY[2]           //6
                        - myFrame.getHeight() / MAGIC_NUMBER_ARRAY[2]); //6
        myFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        myFrame.setLayout(new FlowLayout());
        myFrame.setVisible(true);
        myFrame.requestFocusInWindow();
        myFrame.setResizable(false);
        
        myFrame.add(addOverallLayout());
        myFrame.pack();   
    }
    
    /**
     * This method creates all of the panels required to display the GUI and it's components. 
     * It then adds the various components to the panels to set them to the appropriate state.
     * It creates the panel for the tetris game as well.
     * 
     * @return the JPanel that holds all of the contents of the GUI.
     */
    public JPanel addOverallLayout() {
        
        final JPanel secondFrame = new JPanel(new BorderLayout());
        final JPanel thirdFrame = new JPanel(new FlowLayout());
        final JPanel holdTetris = new JPanel(new FlowLayout());
        
        thirdFrame.setBackground(Color.BLUE.darker().darker());
        holdTetris.setBackground(Color.ORANGE);
        
        secondFrame.add(thirdFrame);
        thirdFrame.add(holdTetris);
        thirdFrame.add(addEastPanel());
        holdTetris.add(myTetris);
        
        secondFrame.add(addMenuBar(), BorderLayout.NORTH);
        myBoard.addObserver(myTetris);
        myTetris.setFocusable(true);

        return secondFrame;
    }
       
    /**
     * This method creates all of the components located on the east side of the GUI. It 
     * creates the panel for viewing the next piece, and the few buttons on the east side.
     * 
     * @return the Panel that holds the content of the east side of the GUI
     */
    private JPanel addEastPanel() {
         
        final NextPiecePanel nextPiece = new NextPiecePanel(myBoard);
        final JPanel eastLayout = new JPanel(new FlowLayout());
        final Box eastBoxLayout = new Box(BoxLayout.Y_AXIS);
        final JPanel buttonPanel = new JPanel();
        final ScorePanel score = new ScorePanel(myBoard, myMoveTimer);
        
        eastLayout.setBackground(Color.BLUE.darker().darker());
        buttonPanel.setLayout(new BoxLayout(buttonPanel, MAGIC_NUMBER_ARRAY[0])); //3
        buttonPanel.setBackground(Color.ORANGE);
        buttonPanel.setPreferredSize(new Dimension(MAGIC_NUMBER_ARRAY[1], 
                                                   MAGIC_NUMBER_ARRAY
                /*100 X 200 Dimension*/             [MAGIC_NUMBER_ARRAY
                                                    [MAGIC_NUMBER_ARRAY[2]]]));

        eastLayout.add(eastBoxLayout);
        eastBoxLayout.add(nextPiece);                                 // vertical strut of 65
        eastBoxLayout.add(Box.createVerticalStrut(MAGIC_NUMBER_ARRAY[MAGIC_NUMBER_ARRAY[0]]));
        eastBoxLayout.add(buttonPanel);
        addButtons(buttonPanel);
        buttonPanel.add(score);
        myBoard.addObserver(score);
        myBoard.addObserver(nextPiece);
        
        return eastLayout;
    }
    
    /**
     * This method adds the JButtons to the buttonPanel on the east side of the GUI.
     * 
     * @param theButtonPanel the panel we add the buttons to
     */
    private void addButtons(final JPanel theButtonPanel) {
        
        final Action pauseAction = new PauseAction(myFrame, myTetris, myMoveTimer, 
                                                                               myKeyListener);
        //make the buttons
        final JButton controlsButton = new JButton(myButtonNameArray[0]);
        final JButton pause = new JButton(myButtonNameArray[1]);
        final JButton endGame = new JButton(myButtonNameArray[2]);
        final JButton newGame = new JButton(myButtonNameArray[myButtonNameArray.length - 2]);
        final JButton infoButton = new JButton(myButtonNameArray[myButtonArray.length - 1]);

        newGame.setEnabled(false);
        myButtonArray[0] = controlsButton;
        myButtonArray[1] = pause;
        myButtonArray[2] = endGame;
        myButtonArray[myButtonNameArray.length - 2] = newGame;
        myButtonArray[myButtonNameArray.length - 1] = infoButton;
        
        for (int i = 0; i < myButtonArray.length; i++) {
            
            myButtonArray[i].addActionListener(myActionListener);
            myButtonArray[i].setActionCommand(myButtonNameArray[i]);
            myButtonArray[i].setFocusable(false);
            theButtonPanel.add(myButtonArray[i]);
        }
        
        myPauseListener = new PauseListener(pause);
        pause.setAction(pauseAction);
        myFrame.addKeyListener(myPauseListener);
    }
    
    /** 
     * This method disables all buttons but the pause button.
     */
    public static void disableAllButPause() {
        
        for (final JButton button : myButtonArray) {
            
            button.setEnabled(false);
        }
        myButtonArray[1].setEnabled(true); // pause
    }
    
    /**
     * This method enables all buttons but the new game button.
     */
    public static void enableAllButNewGame() {
        
        for (final JButton button : myButtonArray) {
            
            button.setEnabled(true);
        }
        myButtonArray[MAGIC_NUMBER_ARRAY[0]].setEnabled(false); // new game index 3
    }
    
    /**
     * This method creates the components located on the menu bar on the top of the GUI. 
     * Primarily right now it contains the buttons used to change the size of the blocks in 
     * the tetris JPanel.
     * 
     * @return the JMenuBar used to contain the JMenuItems for changing size of playing pieces
     */
    private JMenuBar addMenuBar() {
        
        final JMenuBar bar = new JMenuBar();
        final JMenu changeSize = new JMenu("Change Size");
        final JCheckBox grid = new JCheckBox(myStringNamesArray[MAGIC_NUMBER_ARRAY[1]]); 
                                                                              //Grid (index 4)
        //add sizing radio Buttons
        final JRadioButtonMenuItem small = 
            new JRadioButtonMenuItem(myStringNamesArray[1]); //Small
        final JRadioButtonMenuItem normal = 
            new JRadioButtonMenuItem(myStringNamesArray[2]); //Normal
        final JRadioButtonMenuItem large = 
            new JRadioButtonMenuItem(myStringNamesArray[MAGIC_NUMBER_ARRAY[0]]); 
                                                                               //BIG (index 3)
        final ButtonGroup group = new ButtonGroup();
        normal.setSelected(true);
        group.add(small);
        group.add(normal);
        group.add(large);
        
        bar.add(changeSize);
        bar.add(grid);
        changeSize.add(small);
        changeSize.addSeparator();
        changeSize.add(normal);
        changeSize.addSeparator();
        changeSize.add(large);
        
        changeSize.addActionListener(myActionListener);
        grid.addActionListener(myActionListener);
        small.addActionListener(myActionListener);
        normal.addActionListener(myActionListener);
        large.addActionListener(myActionListener);
        
        grid.setActionCommand(grid.getText());
        small.setActionCommand(small.getText());
        normal.setActionCommand(normal.getText());
        large.setActionCommand(large.getText());
        
        bar.setFocusable(false);
        changeSize.setFocusable(false);
        
        return bar;
    }
    
    /**
     * This method instantiates the ChangeControlPanel class and calls one of it's methods to 
     * initiate the dialog box that shows the controls for the game.
     */
    private void changeControls() {
        
        myChangeControls = new ChangeControlsPanel();
        myChangeControls.addPropertyChangeListener(myPauseListener);
        myChangeControls.addPropertyChangeListener(myKeyListener);
        
        myMoveTimer.stop();
        myChangeControls.initiateControlsDialog();
        myMoveTimer.start();
    }  
    
    /**
     * This method calls the methods needed to instantiate the new states of objects required 
     * to start a new game. This method is used when the game ends and the user is prompted 
     * for replaying the game.
     */
    private void newGame() {
        
        myFrame.dispose();
        setup();
        start();
    }
    
    /**
     * This method allows a different area of the GUI to request focus once it is lost.
     * 
     * {@inheritDoc}
     */
    @Override
    public void focusLost(final FocusEvent theEvent) {
        
        myFrame.requestFocusInWindow();
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void focusGained(final FocusEvent theEvent) { /*not used*/ }
 // *********** Inner Class Listeners   *********************************
   
    /**
     * A class that listens for timer events, and action events that correspond to various 
     * JButtons and JMenuItems.
     */
    private class Listener implements ActionListener {
        
        /** The block size of the tetris piece used to change sizes. */
        private final int myBlockSize = TetrisPanel.MY_INITIAL_BLOCK_SIZE;
        
        /**
         * calls methods that control the various listening functions, passing in the action.
         * 
         * @param theEvent The event triggering the action.
         * {@inheritDoc}
         */
        @Override
        public void actionPerformed(final ActionEvent theEvent) {

            final String action = theEvent.getActionCommand();
            
            moveBoardDown(action);
            triggerControlButton(action);
            changeTheSize(action);
            toggleGrid(action);
            endGame(action);
            newGameAction(action);
            scoreInfo(action);
        }
        
        /**
         * This method triggers an action when the timer goes off and moves a piece down. It 
         * also ends the game when the pieces reach the top of the board.
         * 
         * @param theAction The action for the event.
         */
        private void moveBoardDown(final String theAction) {
            
            if (myStringNamesArray[0].equals(theAction)) /*move down*/ {
                
                myBoard.moveDown();

                if (myBoard.isFull()) {
                    
                    myMoveTimer.stop();
                    final int choice = JOptionPane.showConfirmDialog(myTetris, 
                                                                     "Do you want to play "
                                    + "again?", "GAME OVER MAN", JOptionPane.YES_NO_OPTION);
                    
                    if (choice == 1) {
                        
                        myFrame.dispatchEvent(new WindowEvent(myFrame, 
                                                              WindowEvent.WINDOW_CLOSING));
                    } else {
                        
                        myFrame.dispose();
                        newGame();
                    }
                } 
            }
        }
        
        /**
         * This method triggers an action when the control or pause button is clicked.
         * 
         * @param theAction The action for the event.
         */
        private void triggerControlButton(final String theAction) {
            
            if (myButtonNameArray[0].equals(theAction)) /*Change Controls*/ {
                
                myMoveTimer.stop();
                myTetris.removeKeyListener(myKeyListener);
                myFrame.removeKeyListener(myKeyListener);
                myChangeControls.initiateControlsDialog();
                myMoveTimer.start();
                myTetris.addKeyListener(myKeyListener);
                myFrame.addKeyListener(myKeyListener);
            } 
        }
        
        /**
         * This method triggers an action when the size JMenuItems on the top left of the 
         * game are clicked.
         * 
         * @param theAction The action for the event.
         */
        private void changeTheSize(final String theAction) {
            
            if (myStringNamesArray[1].equals(theAction)) /*Small*/ {
                
                myTetris.changeSize(myBlockSize - myBlockSize / MAGIC_NUMBER_ARRAY[0]); //3
                myFrame.pack();
                setScreenLocation(2, MAGIC_NUMBER_ARRAY[0]);  //6
               
            } else if (myStringNamesArray[2].equals(theAction)) /*Normal*/ {
                
                myTetris.changeSize(myBlockSize);
                myFrame.pack();
                setScreenLocation(2, MAGIC_NUMBER_ARRAY[1]); //4
                
            } else if (myStringNamesArray[MAGIC_NUMBER_ARRAY[0]].equals(theAction)) 
                                                                           /*BIG (index 3)*/ {
                
                myTetris.changeSize(myBlockSize + myBlockSize / MAGIC_NUMBER_ARRAY[0]); //3
                myFrame.pack();
                setScreenLocation(2, MAGIC_NUMBER_ARRAY[1]); //4
            } 
        }
        
        /**
         * triggers an action to let TetrisPanel know to turn the grid on.
         * 
         * @param theAction The action for the event.
         */
        private void toggleGrid(final String theAction) {
            
            if (myStringNamesArray[MAGIC_NUMBER_ARRAY[1]].equals(theAction)) 
                                                                         /* Grid (index 4)*/ {
                myTetris.toggleGrid();
                myTetris.repaint();
            }
        }
        
        /**
         * This method repositions the GUI according to the user's screen. 
         * 
         * @param theWidthDivider the number dividing the width of the screen
         * @param theHeightDivider the number dividing the height of the screen
         */
        private void setScreenLocation(final int theWidthDivider, final int theHeightDivider) {
            
            myFrame.setLocation(SCREEN_WIDTH / theWidthDivider 
                                                       - myFrame.getWidth() / theWidthDivider, 
                                SCREEN_HEIGHT / theHeightDivider 
                                                     - myFrame.getHeight() / theHeightDivider);
        }
        
        /**
         * This method allows the game to end when the End Game button is clicked.
         * 
         * @param theAction the action saying end game has been clicked
         */
        private void endGame(final String theAction) {
            
            if (myButtonNameArray[2].equals(theAction)) /*end game*/ {
                
                myTetris.endGame();
                myMoveTimer.removeActionListener(myActionListener);
                myFrame.removeKeyListener(myKeyListener);
                myTetris.repaint();
                
                for (int i = 0; i < myButtonArray.length; i++) {
                    myButtonArray[i].setEnabled(false);                       //all buttons
                    myButtonArray[myButtonArray.length - 2].setEnabled(true); //new game
                }
            }
        }
        
        /**
         * This method allows the user to start a new game when the New Game button is 
         * clicked.
         * 
         * @param theAction the action saying new game has been clicked
         */
        private void newGameAction(final String theAction) {
            
            if (myButtonNameArray[MAGIC_NUMBER_ARRAY[0]].equals(theAction)) /*New Game 
                                                                                 (index 3)*/ { 
                newGame();
                enableAllButNewGame();
            }
        }
        
        /**
         * This method allows a user to click a scoring button to view information about the 
         * scoring process.
         * 
         * @param theAction the action saying the scoring button has been clicked
         */
        private void scoreInfo(final String theAction) {
 
            if (myButtonNameArray[MAGIC_NUMBER_ARRAY[1]].equals(theAction))/*Scoring Info...   
                                                                                (index 4)*/ {
                myMoveTimer.stop();
                myFrame.removeKeyListener(myKeyListener);
                JOptionPane.showMessageDialog(myTetris, 
                                              "You get 250 points per line cleared.\n" 
                    + "Clear 5 or more lines to go up a level.", 
                    theAction, JOptionPane.OK_OPTION);
                
                myMoveTimer.start();
                myFrame.addKeyListener(myKeyListener);
            }
        }
    }
}