/*
 * TCSS 305Winter 2014
 * Assignment 6b-Tetris
 */

package view;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.Line2D;
import java.awt.geom.RoundRectangle2D;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JPanel;

import model.AbstractPiece;
import model.Board;
import model.ImmutablePiece;

/**
 * This class creates the panel that holds the tetris game screen, and allows the game to be 
 * drawn on the panel.
 * 
 * @author Brian Dennis
 * @version 7 March 2014
 */
@SuppressWarnings("serial")
public class TetrisPanel extends JPanel implements Observer {

    /** The inital width and height of a tetris piece. */
    public static final int MY_INITIAL_BLOCK_SIZE = 25;
    
    /** the number representing how much curve the squares that make up the pieces will have.*/
    private static final int CURVE_NUMBER = 15;
    
    /** the board that contains the logic behind the coordinates of the pieces. */
    private final Board myBoard;
    
    /** the width and height of the tetris pieces that can change due to resizing. */
    private int myBlockSize; 
    
    /** The boolean that keeps track of if the game has ended. */
    private boolean myGameIsEnded;
    
    /** The boolean that keeps track of if the is paused. */
    private boolean myGameIsPaused;
    
    /** The boolean that keeps track of if the grid is on. */
    private boolean myGridIsOn;
    
    /**
     * The constructor of the panel which initializes variables, sizes and background colors.
     * @param theBoard the reference of the playing board of the game
     */
    public TetrisPanel(final Board theBoard) {
  
        super();
        myBoard = theBoard;
        myBlockSize = MY_INITIAL_BLOCK_SIZE;
        setPreferredSize(new Dimension(myBoard.getWidth() * myBlockSize, 
                                      myBoard.getHeight() * myBlockSize));
        myGameIsEnded = false;
        myGameIsPaused = false;
    }

        /**
         * Paints this panel on the screen with the specified Graphics object. Calling methods 
         * to draw pieces.
         * 
         * @param theGraphics The Graphics object.
         */
    @Override
    public void paintComponent(final Graphics theGraphics) {
            
        super.paintComponent(theGraphics);
        final Graphics2D g2 = (Graphics2D) theGraphics;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                                                           RenderingHints.VALUE_ANTIALIAS_ON);
        setBackground(Color.WHITE);
        drawFrozenPieces(g2);
        drawFallingPieces(g2);
        
        if (myGridIsOn) {
            
            drawGrid(g2);
        }
        
        if (myGameIsEnded) {
            
            setBackground(Color.BLACK);
            g2.setColor(Color.WHITE);
            g2.drawString("GAME OVER", getWidth() / 2, getHeight() / 2);
            
        }  else if (myGameIsPaused) {
            
            setBackground(Color.GRAY);
            g2.setColor(Color.WHITE);
            g2.drawString("PAUSED", getWidth() / 2, getHeight() / 2);
        }
    }
        
    /**
     * This method draws the frozen pieces.
     * 
     * @param theGraphics The Graphics object
     */
    private void drawFrozenPieces(final Graphics2D theGraphics) {
            
        for (int y = 0; y <= myBoard.getHeight() - 1; y++) {
                
            for (int x = 0; x <= myBoard.getWidth() - 1; x++) {

                if (myBoard.getRowAt(y)[x] != null) {
                              
                    theGraphics.setPaint(myBoard.getRowAt(y)[x]);                   
                    theGraphics.fill(new RoundRectangle2D.Double(x * myBlockSize, 
                                                getHeight() - (y * myBlockSize) - myBlockSize,
                                       myBlockSize, myBlockSize, CURVE_NUMBER, CURVE_NUMBER));
                    theGraphics.setStroke(new BasicStroke(2));
                    theGraphics.setPaint(Color.BLACK);
                    theGraphics.draw(new RoundRectangle2D.Double(x * myBlockSize, 
                                                getHeight() - (y * myBlockSize) - myBlockSize,
                                       myBlockSize, myBlockSize, CURVE_NUMBER, CURVE_NUMBER));
                }     
            }
        } 
    }
        
    /**
     * This method draws the falling pieces.
     * 
     * @param theGraphics the graphics object
     */
    private void drawFallingPieces(final Graphics2D theGraphics) {
            
        final ImmutablePiece piece = (AbstractPiece) myBoard.getCurrentPiece();
            
        for (int i = 0; i < ((AbstractPiece) piece).getPoints().length; i++) {
                
            final Point point = new Point(
                                       ((AbstractPiece) piece).getAbsolutePosition(i).getX(),  
                                       ((AbstractPiece) piece).getAbsolutePosition(i).getY());
            theGraphics.setPaint(((AbstractPiece) piece).getColor());
            theGraphics.fill(new RoundRectangle2D.Double(point.getX() * myBlockSize, 
                                     getHeight() - (point.getY() * myBlockSize + myBlockSize),
                                       myBlockSize, myBlockSize, CURVE_NUMBER, CURVE_NUMBER));
            theGraphics.setStroke(new BasicStroke(2));
            theGraphics.setPaint(Color.BLACK);
            theGraphics.draw(new RoundRectangle2D.Double(point.getX() * myBlockSize, 
                                     getHeight() - (point.getY() * myBlockSize + myBlockSize),
                                       myBlockSize, myBlockSize, CURVE_NUMBER, CURVE_NUMBER));
        }
    }
    
    /**
     * This method draws the grid on the panel.
     * @param theGraphics the graphics object
     */
    private void drawGrid(final Graphics2D theGraphics) {
            
        theGraphics.setStroke(new BasicStroke(1));
        theGraphics.setPaint(Color.BLACK);
        for (int i = 0; i < getWidth(); i += myBlockSize) {
            final Shape line = new Line2D.Float(i, 0, i, getHeight());
            theGraphics.draw(line);
        }

        for (int i = 0; i < getHeight(); i += myBlockSize) {
            final Shape line = new Line2D.Float(0, i, getWidth(), i);
            theGraphics.draw(line);
        }
    }
        
    /**
     * This method will change the size to the new size. There are 3 different choices.
     * 
     * @param theBlockSize the new size of the Frame
     */
    public void changeSize(final int theBlockSize) {
              
        
        myBlockSize = theBlockSize;
    
        setPreferredSize(new Dimension(myBoard.getWidth() * myBlockSize, 
                                                          myBoard.getHeight() * myBlockSize));
    }
    
    /**
     * This method changes the boolean myGameIsEnded to true to let the game know to end 
     * itself.
     */
    public void endGame() {
        
        myGameIsEnded = true;  
    }
    
    /**
     * This method toggles the boolean myGameIsPause to let the game know to pause itself.
     */
    public void togglePause() {
        
        myGameIsPaused ^= true;
    }
    
    /**
     * This method toggles the boolean myGridIsOn to let the game know to turn the grid on. 
     */
    public void toggleGrid() {
        
        myGridIsOn ^= true;
    }
     
    /**
     * All this method does is update the graphics panel when called by the observable class.
     * {@inheritDoc}
     */
    @Override
    public void update(final Observable theBoard, final Object theArg) {
   
        repaint();
    }
}